
from argparse import ArgumentParser
from argparse import RawTextHelpFormatter
from sys import argv
from sys import exit
from sys import stderr

import shukujitsu


VERSION = """\
%(prog)s v{version}

MIT license <https://opensource.org/licenses/MIT>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.

Written by Youhei Sakurai and others; see
<https://github.com/sakurai-youhei/python-shukujitsu/graphs/contributors>.
""".format(version=shukujitsu.__version__)


def getargs(args):
    jp_holidays = shukujitsu.Japan()
    parser = ArgumentParser(prog="shukujitsu",
                            formatter_class=RawTextHelpFormatter,
                            description="Select Japanese holidays from %d to "
                                        "%d" % (min(jp_holidays).year,
                                                max(jp_holidays).year),
                            epilog="With no DATE, read standard input. Exit "
                                   "status is 0 if any date is selected, 1 "
                                   "otherwise.")
    parser.add_argument("-i", "--invert-match", action="store_true",
                        help="select non-holidays")
    parser.add_argument("-n", "--holiday-name", action="store_true",
                        help="output holiday name instead")
    parser.add_argument("-V", "--version", action="version", version=VERSION,
                        help="display version information and exit")
    parser.add_argument("dates", metavar="DATE", type=str, nargs="*",
                        help="date to be examined")
    return parser.parse_args(args)


def main():
    args = getargs(argv[1:])
    status = 1
    try:
        if args.invert_match and args.holiday_name:
            raise RuntimeError("Both options can't be enabled, --invert-match "
                               "and --holiday-name.")
        jp_holidays = shukujitsu.Japan()
        for date in args.dates and args.dates or iter(input, None):
            try:
                key = int(date)
            except ValueError:
                key = date
            if args.invert_match:
                if key not in jp_holidays:
                    print(date.strip())
                    status = 0
            elif key in jp_holidays:
                if args.holiday_name:
                    print(jp_holidays[key])
                else:
                    print(date.strip())
                status = 0
    except (EOFError, KeyboardInterrupt):
        pass
    except Exception as e:
        print(e, file=stderr)
    finally:
        exit(status)


if __name__ == "__main__":
    main()



from importlib import import_module
from warnings import warn


def load_txt(*args, **kwargs):
    return load("TXT", *args, **kwargs)


def load_bin(*args, **kwargs):
    return load("BIN", *args, **kwargs)


_DEFAULT_BUNDLES = """\
asof20230201
asof20220201
asof20210201
asof20201127
asof20201118
""".split()


def load(type_, bundles=_DEFAULT_BUNDLES):
    for asof in bundles:
        try:
            mod = import_module("shukujitsu.%s" % asof)
        except ImportError as e:
            warn(repr(e), category=ImportWarning, stacklevel=2)
        else:
            return getattr(mod, type_)
    else:
        raise RuntimeError("No holidays data is available")



from pkgutil import get_data

# https://www8.cao.go.jp/chosei/shukujitsu/shukujitsu.csv
BIN = get_data("shukujitsu.asof20230201", "shukujitsu.csv")
if BIN:
    TXT = BIN.decode("cp932")
else:
    raise ImportError("Can't load shukujitsu.csv under {}".format(__name__))



from collections import ChainMap

from shukujitsu.holidays import JapaneseHolidays


def Japan(years=[]):
    holidays = JapaneseHolidays()
    if years:
        maps = (holidays._year(int(yyyy)) for yyyy in years)
        holidays = JapaneseHolidays(ChainMap(*maps))
    return holidays


def CountryHoliday(country, years=[], *args, **kwargs):
    if any(args) or any(kwargs):
        raise RuntimeError("Unsupported arguments provided, %r %r" % (args,
                                                                      kwargs))
    elif country not in "Japan JP JPN".split():
        raise RuntimeError("Unsupported country provided, %s" % country)
    return Japan(years)
